%Aaron M. Schinder
%9 Aug 2015
%create_mesh - creates a triangle mesh from a gear curve
%Inputs:
%gearx - list of x coordinates of gear outer edge
%geary - list of y coordinates of gear outer edge
%shaftr - inner shaft radius
%hgt - height of gear
%Outputs: [tris,x,y,z]
%tris - triangle list
%x,y,z - coordinates of nodes
function [tris,x,y,z] = create_mesh(gearx,geary,shaftr,hgt)

for I = 1:size(gearx,2)
    th(I) = arg(gearx(I),geary(I));
    xshft(I) = shaftr*cos(th(I));
    yshft(I) = shaftr*sin(th(I));
end

Atol = 1E-20;

tris = [];
x = [];
y = [];
z = [];

K = 1; %point count
L = 1; %tri count
for I = 1:(size(gearx,2))
    if(I<size(gearx,2))
        pt1x = xshft(I);
        pt1y = yshft(I);
        pt2x = xshft(I+1);
        pt2y = yshft(I+1);
        pt3x = gearx(I+1);
        pt3y = geary(I+1);
        pt4x = gearx(I);
        pt4y = geary(I);
    else
        pt1x = xshft(I);
        pt1y = yshft(I);
        pt2x = xshft(1);
        pt2y = yshft(1);
        pt3x = gearx(1);
        pt3y = geary(1);
        pt4x = gearx(I);
        pt4y = geary(I);
        
    end

    %bottom surface
    K0 = K;
    x(K) = pt1x;
    y(K) = pt1y;
    z(K) = 0;
    K = K + 1;
    x(K) = pt2x;
    y(K) = pt2y;
    z(K) = 0;
    K = K + 1;
    x(K) = pt3x;
    y(K) = pt3y;
    z(K) = 0;
    K = K + 1;
    x(K) = pt4x;
    y(K) = pt4y;
    z(K) = 0;
    K = K + 1;
    
    v1 = [x(K0+1)-x(K0+0),y(K0+1)-y(K0+0),z(K0+1)-z(K0+0)];
    v2 = [x(K0+2)-x(K0+1),y(K0+2)-y(K0+1),z(K0+2)-z(K0+1)];
    if(norm(cross(v1,v2))>Atol)
        tt = [K0,K0+1,K0+2];
        tris(L,:) = tt;
        L = L + 1;
    end
    
    v1 = [x(K0+3)-x(K0+2),y(K0+3)-y(K0+2),z(K0+3)-z(K0+2)];
    v2 = [x(K0+0)-x(K0+3),y(K0+0)-y(K0+3),z(K0+0)-z(K0+3)];
    if(norm(cross(v1,v2))>Atol)
        tt = [K0+2,K0+3,K0+0];
        tris(L,:) = tt;
        L = L + 1;
    end
    
    %top surface
    K0 = K;
    x(K) = pt1x;
    y(K) = pt1y;
    z(K) = hgt;
    K = K + 1;
    x(K) = pt2x;
    y(K) = pt2y;
    z(K) = hgt;
    K = K + 1;
    x(K) = pt3x;
    y(K) = pt3y;
    z(K) = hgt;
    K = K + 1;
    x(K) = pt4x;
    y(K) = pt4y;
    z(K) = hgt;
    K = K + 1;
    
    v1 = [x(K0+1)-x(K0+0),y(K0+1)-y(K0+0),z(K0+1)-z(K0+0)];
    v2 = [x(K0+2)-x(K0+1),y(K0+2)-y(K0+1),z(K0+2)-z(K0+1)];
    if(norm(cross(v1,v2))>Atol)
        tt = [K0+1,K0,K0+2];
        tris(L,:) = tt;
        L = L + 1;
    end
    
    v1 = [x(K0+3)-x(K0+2),y(K0+3)-y(K0+2),z(K0+3)-z(K0+2)];
    v2 = [x(K0+0)-x(K0+3),y(K0+0)-y(K0+3),z(K0+0)-z(K0+3)];
    if(norm(cross(v1,v2))>Atol)
        tt = [K0+3,K0+2,K0+0];
        tris(L,:) = tt;
        L = L + 1;
    end
    
    %inner surface
    K0 = K;
    x(K) = pt1x;
    y(K) = pt1y;
    z(K) = 0;
    K = K + 1;
    x(K) = pt2x;
    y(K) = pt2y;
    z(K) = 0;
    K = K + 1;
    x(K) = pt2x;
    y(K) = pt2y;
    z(K) = hgt;
    K = K + 1;
    x(K) = pt1x;
    y(K) = pt1y;
    z(K) = hgt;
    K = K + 1;
    
    v1 = [x(K0+1)-x(K0+0),y(K0+1)-y(K0+0),z(K0+1)-z(K0+0)];
    v2 = [x(K0+2)-x(K0+1),y(K0+2)-y(K0+1),z(K0+2)-z(K0+1)];
    if(norm(cross(v1,v2))>Atol)
        tt = [K0,K0+1,K0+2];
        tris(L,:) = tt;
        L = L + 1;
    end
    
    v1 = [x(K0+3)-x(K0+2),y(K0+3)-y(K0+2),z(K0+3)-z(K0+2)];
    v2 = [x(K0+0)-x(K0+3),y(K0+0)-y(K0+3),z(K0+0)-z(K0+3)];
    if(norm(cross(v1,v2))>Atol)
        tt = [K0+2,K0+3,K0+0];
        tris(L,:) = tt;
        L = L + 1;
    end
    
    %outer surface
    K0 = K;
    x(K) = pt4x;
    y(K) = pt4y;
    z(K) = 0;
    K = K + 1;
    x(K) = pt3x;
    y(K) = pt3y;
    z(K) = 0;
    K = K + 1;
    x(K) = pt3x;
    y(K) = pt3y;
    z(K) = hgt;
    K = K + 1;
    x(K) = pt4x;
    y(K) = pt4y;
    z(K) = hgt;
    K = K + 1;
    
    v1 = [x(K0+1)-x(K0+0),y(K0+1)-y(K0+0),z(K0+1)-z(K0+0)];
    v2 = [x(K0+2)-x(K0+1),y(K0+2)-y(K0+1),z(K0+2)-z(K0+1)];
    if(norm(cross(v1,v2))>Atol)
        tt = [K0,K0+1,K0+2];
        tris(L,:) = tt;
        L = L + 1;
    end
    
    v1 = [x(K0+3)-x(K0+2),y(K0+3)-y(K0+2),z(K0+3)-z(K0+2)];
    v2 = [x(K0+0)-x(K0+3),y(K0+0)-y(K0+3),z(K0+0)-z(K0+3)];
    if(norm(cross(v1,v2))>Atol)
        tt = [K0+2,K0+3,K0+0];
        tris(L,:) = tt;
        L = L + 1;
    end
    
end

return

