%Aaron M. Schinder
%9 Aug 2015
%create_stl(Fname,name,gearx,geary,shaftr,hgt,scale)
%using an x,y profile for a gear, creates an STL file for that gear
%with a shaft of a given radius, and a given height
%Inputs:
%Fname - name of stl output file (ASCII)
%name - name of solid object
%gearx - gear x profile
%geary - gear y profile
%shaftr - shaft radius
%hgt - height of gear
%scale - scale factor (25.4 to convert from inches (ANSI) to mm)
function create_stl(Fname,name,gearx,geary,shaftr,hgt,scale)
[tri,xx,yy,zz] = create_mesh(gearx,geary,shaftr,hgt);
for I = 1:size(xx,2)
    xx(I) = xx(I)*scale;
    yy(I) = yy(I)*scale;
    zz(I) = zz(I)*scale;
end

fp = fopen(Fname,'w+');
fprintf(fp,'solid %s\n',name);

for I = 1:size(tri,1)
    vx1 = xx(tri(I,1));vy1 = yy(tri(I,1));vz1 = zz(tri(I,1));
    vx2 = xx(tri(I,2));vy2 = yy(tri(I,2));vz2 = zz(tri(I,2));
    vx3 = xx(tri(I,3));vy3 = yy(tri(I,3));vz3 = zz(tri(I,3));
    v1 = [vx2-vx1,vy2-vy1,vz2-vz1];
    v2 = [vx3-vx2,vy3-vy2,vz3-vz2];
    nn = cross(v2,v1);
    nm = norm(nn);
    nn2 = nn./nm;
    
    fprintf(fp,'facet normal %1.6g %1.6g %1.6g\n',nn2(1),nn2(2),nn2(3));
    fprintf(fp,'\touter loop\n');
    fprintf(fp,'\t\tvertex %1.6g %1.6g %1.6g\n',vx1,vy1,vz1);
    fprintf(fp,'\t\tvertex %1.6g %1.6g %1.6g\n',vx2,vy2,vz2);
    fprintf(fp,'\t\tvertex %1.6g %1.6g %1.6g\n',vx3,vy3,vz3);
    fprintf(fp,'\tend loop\n');
    fprintf(fp,'end facet\n');
end





fclose(fp);

return
